unit CompilerDeklarationen;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;


const
  { ********************************************************** }
  { ***  wichtige Konstanten fuer Listen und Recordlaengen *** }
  { ********************************************************** }

  LEN_SI_ARR               = 900000;

  // Laenge de Statement-Liste
  LEN_STM_LISTE            = 50000;

  // Laenge der Name-Tabelle
  LEN_NAME_TAB             = 9999;

  // Laenge der Procedure-Tabelle
  LEN_PROC_LISTE           = 999;


  Laenge_While_Liste       =  50;
  Lange_IF_THEN_Liste      =  50;
  Laenge_FTD_Liste         =  50;
  Laenge_CASE_Liste        =  50;
  Laenge_VAR_ARRAY_Liste   =  50;
  Laenge_Bit_Liste         = 100;
  Laenge_CONST_ARRAY_Liste =  50;
  Laenge_CONST_TERM_Liste  =  50; // Anzahl der Term-Elemente bei der Definition von Kostanten
  Laenge_akt_CALL_Liste    =  10;
  Laenge_PROC_PAR_Liste    =  10; // Die Anzahl der Prozedur-Parameter ist durch die Anzahl der
                                  // Register begrenzt (mehr als 10 sind nicht sinnvoll)

  Laenge_ISR_Liste         =  200;

type



  { *** SCANNER *** }

  // Scan-Item-Typ
  SI_Typ = (kein_SI, SI_Zahl, SI_Zeichen, SI_Bezeichner, SI_Zeichenkette);

  SI_Unter_Typ = (kein_UT,
                  { ## Untertyp fuer Zahl ## }
                  UT_Dezi_Zahl, UT_Bin_Zahl, UT_Hex_Zahl, UT_Char_Zahl, UT_Curr_Zahl,
                  { ## Untertyp fuer Char ## }
                  UT_Char,
                  { ## Untertyp fuer Zeichenkette ## }
                  UT_String,
                  { ## Untertyp fuer Zeichen ## }
                  UT_Zeichen_Anfang,
                  UT_Zei_DZ_KG { <= }, UT_Zei_DZ_GG { >= }, UT_Zei_DZ_UG { <> },
                  UT_Zei_DZ_DG { := }, UT_Zei_PLUS { + }, UT_Zei_MINUS { - },
                  UT_Zei_MAL { * }, UT_Zei_DURCH { / }, UT_Zei_SEMI { ; },
                  UT_Zei_DP { : }, UT_Zei_KOMMAR { , }, UT_Zei_PUNKT { . },
                  UT_Zei_GLEICH { = }, UT_Zei_KA { ( }, UT_Zei_KZ { ) },
                  UT_Zei_KLEINER { < }, UT_Zei_GROESSER { > },
                  UT_Zei_EKA { [ }, UT_Zei_EKZ { ] }, UT_Zei_SS { | },
                  UT_Zei_PUNKT_PUNKT { .. },
                  UT_Zeichen_Ende,
                  { ## Untertyp fuer Bezeichner ## }
                  UT_Bezeichner_PAR,
                  UT_Bezeichner_Anfang,
                  UT_Bez_PROGRAM, UT_Bez_DEVICE, UT_Bez_IMPORT, UT_Bez_DEFINE,
                  UT_Bez_IMPLEMENTATION,
                  UT_Bez_END, UT_Bez_CONST, UT_Bez_VAR, UT_Bez_BEGIN,
                  UT_Bez_TRUE, UT_Bez_FALSE, UT_Bez_PROCEDURE, UT_Bez_FUNCTION,
                  UT_Bez_ASM, UT_Bez_ENDASM, UT_Bez_RESULT,
                  UT_Bez_FOR, UT_Bez_TO, UT_Bez_DOWNTO, UT_Bez_DO, UT_Bez_ENDFOR,
                  UT_Bez_LOOP, UT_Bez_ENDLOOP,
                  UT_Bez_IF, UT_Bez_THEN, UT_Bez_ELSE, UT_Bez_ENDIF,
                  UT_Bez_INC, UT_Bez_DEC, UT_Bez_SHL, UT_Bez_SHR, UT_Bez_NOP,
                  UT_Bez_SETBIT, UT_Bez_CLEARBIT, UT_Bez_BITHIGH, UT_Bez_BITLOW,
                  UT_Bez_DIV, UT_Bez_MOD, UT_Bez_IN, UT_Bez_AND, UT_Bez_OR,
                  UT_Bez_ARRAY, UT_Bez_OF,
                  UT_Bez_BYTETOWORD, UT_Bez_HIGH, UT_Bez_LOW,
                  UT_Bez_WHILE, UT_Bez_ENDWHILE,
                  UT_Bez_CASE, UT_Bez_ENDCASE,
                  UT_Bez_BYTETOCHAR, UT_Bez_CHARTOBYTE, UT_Bez_LENGTH,
                  UT_Bez_DWORDTOBYTE, UT_Bez_DWLOW, UT_Bez_DWHIGH,
                  UT_Bez_WORDTODWORD, UT_Bez_BYTETODWORD,
                  UT_Bez_ISR, UT_Bez_EnableInts, UT_Bez_DisableInts, UT_Bez_ISR_LISTE,

                  UT_Bezeichner_Ende,
                  { ## Untertyp fuer Parser ## }
                  UT_Par_prog_name_rd, UT_Par_prog_device, UT_Par_prog_kopf, UT_Par_prog_jump,
                  UT_Par_prog_name_cp,

                  UT_Par_call_bez,

                  UT_Par_var_name, UT_Par_var_liste, UT_Par_var_type,
                  UT_Par_var_bit_reg, UT_Par_var_bit_pos, UT_Par_var_bit_type,
                  UT_Par_var_array_von, UT_Par_var_array_bis,
                  UT_Par_var_array_2d_von, UT_Par_var_array_2d_bis,
                  UT_Par_var_array_type,
                  UT_Par_var_string, UT_Par_var_str_len, UT_Par_var_ende,

                  UT_Par_const_name, UT_Par_const_type, UT_Par_const_ende,
                  UT_Par_const_wert, UT_Par_const_bez, UT_Par_const_operator,
                  UT_Par_const_array_von, UT_Par_const_array_bis,
                  UT_Par_const_array_2d_von, UT_Par_const_array_2d_bis,
                  UT_Par_const_array_type,
                  UT_Par_const_array_element, UT_Par_const_array_end,

                  UT_Par_zuw_Ziel,
                  UT_Par_zuw_Z_V_ARR_1D, UT_Par_zuw_Z_V_ARR_POS_1D,
                  UT_Par_zuw_Z_V_ARR_2D, UT_Par_zuw_Z_V_ARR_POS_2D_01,
                  UT_Par_zuw_Z_V_ARR_POS_2D_02,
                  UT_Par_zuw_Z_WAA_ADR, UT_Par_zuw_Z_WAA_ADR_POS,
                  UT_Par_zuw_fak_VAR, UT_Par_zuw_fak_CONST, UT_Par_zuw_fak_CONRD,
                  UT_Par_zuw_fak_FKT, UT_Par_zuw_fak_CON_ARR,

                  UT_Par_zuw_Q_V_ARR_1D, UT_Par_zuw_Q_V_ARR_POS_1D,

                  UT_Par_zuw_Q_V_ARR_2D, UT_Par_zuw_Q_V_ARR_POS_2D_01,
                  UT_Par_zuw_Q_V_ARR_POS_2D_02,

                  UT_Par_zuw_operator, UT_Par_zuw_abschl,
                  UT_Par_zuw_fak_B2W, UT_Par_zuw_fak_HIGH, UT_Par_zuw_fak_LOW,
                  UT_Par_zuw_fak_BITHIGH, UT_Par_zuw_fak_BITLOW,
                  UT_Par_zuw_fak_B2C, UT_Par_zuw_fak_C2B, UT_Par_zuw_fak_LENGTH,
                  UT_Par_zuw_fak_DWBYTE1, UT_Par_zuw_fak_DWBYTE2,
                  UT_Par_zuw_fak_DWLOW, UT_Par_zuw_fak_DWHIGH,
                  UT_Par_zuw_fak_WORDTODW, UT_Par_zuw_fak_BYTETODW,
                  UT_Par_zuw_fak_ARRTOADR, UT_Par_zuw_fak_RDADRARR,


                  UT_Par_proc_name_rd, UT_Par_proc_par_name_rd, UT_Par_proc_par_type_rd,
                  UT_Par_proc_par_liste_rd, UT_Par_proc_name_cp,
                  UT_Par_proc_lvar_name_rd, UT_Par_proc_lvar_liste_rd, UT_Par_proc_lvar_type_rd,

                  UT_Par_proc_call_name, UT_Par_proc_call_par, UT_Par_proc_call_par_lst,
                  UT_Par_proc_call_exec,

                  UT_Par_asm_start,

                  UT_Par_func_name_rd,
                  UT_Par_func_name_cp, UT_Par_func_result_type_rd,
                  UT_Par_func_result_rd, UT_Par_func_end, UT_Par_func_call_exec,

                  UT_Par_ftd_ziel, UT_Par_ftd_vergl, UT_Par_ftd_loop,
                  UT_Par_ftd_to, UT_Par_ftd_downto,

                  UT_Par_loop_start, UT_Par_loop_end,

                  UT_Par_if_then_init, UT_Par_if_then_start, UT_Par_if_then_else, UT_Par_if_then_end,

                  UT_Par_inc, UT_Par_dec, UT_Par_shl, UT_Par_shr, UT_Par_nop,

                  UT_Par_setbit, UT_Par_clearbit,

                  UT_Par_while_init, UT_Par_while_start, UT_Par_while_end,

                  UT_Par_case_var, UT_Par_case_element, UT_Par_case_ele_ein, UT_Par_case_ele_ber,
                  UT_Par_case_ele_lis, UT_Par_case_ele_lis_end,

                  UT_Par_case_else, UT_Par_case_end,

                  UT_Par_isr_name_rd, UT_Par_isr_name_cp,
                  UT_Par_EnableInts, UT_Par_DisableInts, UT_Par_isrl_nr,
                  UT_Par_isrl_org, UT_Par_isrl_isr, UT_Par_isrl_bez,

                  UT_ende

                  );


  SI_REC = record
    datei, zeile, spalte : Integer;
    typ : SI_Typ;
    Unter_Typ : SI_Unter_Typ;
    parameter : String;
  end;


  BEZ_REC = record
    bezeichner : String;
    UT_Bez : SI_Unter_Typ;
  end;


  { *** PARSER *** }


const

  Length_Color_List = 1000;

  // Farben
  cl_dunkel_gruen = $009500;
  cl_dunkel_blau  = $650106;
  cl_dunkel_rot   = $0000CA;
  cl_dunkel_braun = $000066;


  LEN_BEZ_CONST = 63;

  BEZ_CONST : array[1..LEN_BEZ_CONST] of BEZ_REC =
      (
  { 01 } (bezeichner : 'PROGRAM';   UT_Bez : UT_Bez_PROGRAM),
  { 02 } (bezeichner : 'END';       UT_Bez : UT_Bez_END),
  { 03 } (bezeichner : 'CONST';     UT_Bez : UT_Bez_CONST),
  { 04 } (bezeichner : 'VAR';       UT_Bez : UT_Bez_VAR),
  { 05 } (bezeichner : 'BEGIN';     UT_Bez : UT_Bez_BEGIN),
  { 06 } (bezeichner : 'TRUE';      UT_Bez : UT_Bez_TRUE),
  { 07 } (bezeichner : 'FALSE';     UT_Bez : UT_Bez_FALSE),
  { 08 } (bezeichner : 'PROCEDURE'; UT_Bez : UT_Bez_PROCEDURE),
  { 09 } (bezeichner : 'FUNCTION';  UT_Bez : UT_Bez_FUNCTION),
  { 10 } (bezeichner : 'ASM';       UT_Bez : UT_Bez_ASM),
  { 11 } (bezeichner : 'ENDASM';    UT_Bez : UT_Bez_ENDASM),
  { 12 } (bezeichner : 'RESULT';    UT_Bez : UT_Bez_RESULT),

  { 13 } (bezeichner : 'FOR';       UT_Bez : UT_Bez_FOR),
  { 14 } (bezeichner : 'TO';        UT_Bez : UT_Bez_TO),
  { 15 } (bezeichner : 'DOWNTO';        UT_Bez : UT_Bez_DOWNTO),

  { 16 } (bezeichner : 'DO';        UT_Bez : UT_Bez_DO),
  { 17 } (bezeichner : 'ENDFOR';    UT_Bez : UT_Bez_ENDFOR),

  { 18 } (bezeichner : 'LOOP';      UT_Bez : UT_Bez_LOOP),
  { 19 } (bezeichner : 'ENDLOOP';   UT_Bez : UT_Bez_ENDLOOP),

  { 20 } (bezeichner : 'DEVICE';    UT_Bez : UT_Bez_DEVICE),
  { 21 } (bezeichner : 'IMPORT';    UT_Bez : UT_Bez_IMPORT),
  { 22 } (bezeichner : 'DEFINE';    UT_Bez : UT_Bez_DEFINE),
  { 23 } (bezeichner : 'IMPLEMENTATION'; UT_Bez : UT_Bez_IMPLEMENTATION),

  { 24 } (bezeichner : 'IF';        UT_Bez : UT_Bez_IF),
  { 25 } (bezeichner : 'THEN';      UT_Bez : UT_Bez_THEN),
  { 26 } (bezeichner : 'ELSE';      UT_Bez : UT_Bez_ELSE),
  { 27 } (bezeichner : 'ENDIF';     UT_Bez : UT_Bez_ENDIF),

  { 28 } (bezeichner : 'INC';       UT_Bez : UT_Bez_INC),
  { 29 } (bezeichner : 'DEC';       UT_Bez : UT_Bez_DEC),
  { 30 } (bezeichner : 'SHL';       UT_Bez : UT_Bez_SHL),
  { 31 } (bezeichner : 'SHR';       UT_Bez : UT_Bez_SHR),
  { 32 } (bezeichner : 'NOP';       UT_Bez : UT_Bez_NOP),

  { 33 } (bezeichner : 'DIV';       UT_Bez : UT_Bez_DIV),
  { 34 } (bezeichner : 'MOD';       UT_Bez : UT_Bez_MOD),
  { 35 } (bezeichner : 'IN';        UT_Bez : UT_Bez_IN),
  { 36 } (bezeichner : 'AND';       UT_Bez : UT_Bez_AND),
  { 37 } (bezeichner : 'OR';        UT_Bez : UT_Bez_OR),

  { 38 } (bezeichner : 'ARRAY';         UT_Bez : UT_Bez_ARRAY),
  { 39 } (bezeichner : 'OF';            UT_Bez : UT_Bez_OF),

  { 40 } (bezeichner : 'SETBIT';        UT_Bez : UT_Bez_SETBIT),
  { 41 } (bezeichner : 'CLEARBIT';      UT_Bez : UT_Bez_CLEARBIT),
  { 42 } (bezeichner : 'BITHIGH';       UT_Bez : UT_Bez_BITHIGH),
  { 43 } (bezeichner : 'BITLOW';        UT_Bez : UT_Bez_BITLOW),

  { 44 } (bezeichner : 'BYTETOWORD';    UT_Bez : UT_Bez_BYTETOWORD),
  { 45 } (bezeichner : 'HIGH';          UT_Bez : UT_Bez_HIGH),
  { 46 } (bezeichner : 'LOW';           UT_Bez : UT_Bez_LOW),

  { 47 } (bezeichner : 'WHILE';         UT_Bez : UT_Bez_WHILE),
  { 48 } (bezeichner : 'ENDWHILE';      UT_Bez : UT_Bez_ENDWHILE),

  { 49 } (bezeichner : 'CASE';          UT_Bez : UT_Bez_CASE),
  { 50 } (bezeichner : 'ENDCASE';       UT_Bez : UT_Bez_ENDCASE),


  { 51 } (bezeichner : 'BYTETOCHAR';    UT_Bez : UT_Bez_BYTETOCHAR),
  { 52 } (bezeichner : 'CHARTOBYTE';    UT_Bez : UT_Bez_CHARTOBYTE),

  { 53 } (bezeichner : 'LENGTH';        UT_Bez : UT_Bez_LENGTH),

  { 54 } (bezeichner : 'DWORDTOBYTE';   UT_Bez : UT_Bez_DWORDTOBYTE),
  { 55 } (bezeichner : 'DWLOW';         UT_Bez : UT_Bez_DWLOW),
  { 56 } (bezeichner : 'DWHIGH';        UT_Bez : UT_Bez_DWHIGH),
  { 57 } (bezeichner : 'WORDTODWORD';   UT_Bez : UT_Bez_WORDTODWORD),
  { 58 } (bezeichner : 'BYTETODWORD';   UT_Bez : UT_Bez_BYTETODWORD),

  { 59 } (bezeichner : 'ISR';           UT_Bez : UT_Bez_ISR),

  { 60 } (bezeichner : 'ENABLEINTS';    UT_Bez : UT_Bez_EnableInts),
  { 61 } (bezeichner : 'DISABLEINTS';   UT_Bez : UT_Bez_DisableInts),

  { 62 } (bezeichner : 'ISR_LISTE';     UT_Bez : UT_Bez_ISR_LISTE),

  { 63 } (bezeichner : '';          UT_Bez : kein_UT)
       );


const
  RecArray_LEN = 6;

type
  RW_Typ = (RW_UND_REC,  { Regelwerk "UND" Record }
            RW_0_1_REC, { Regelwerk "0-1" Record - mindestens 0 - maximal 1 }
            RW_ODER_SCH_0_N_REC, { Regelwerk "ODER-SCHLEIFE" Record - mindestens 0 - maximal unendlich }
            RW_ODER_REC { Regelwerk "ODER-SCHLEIFE" Record - es muss ein Element richtig sein }
            );

  // RB -> Regelwerk-Bezeichner
  RW_Bez = (RB_keiner, RB_prog, RB_prog2, RB_prog3, RB_element, RB_var_element, RB_var_ele_sch,
            RB_var_ele_sch2, RB_var_ele_lst, RB_var_ele_lst2,

            RB_var_varianten, RB_var_std, RB_var_bit, RB_var_bit2,
            RB_var_array, RB_var_array2,
            RB_var_array_1D, RB_var_array_2D,
            RB_var_std2, RB_var_std3,
            RB_var_string,

            RB_const_element, RB_const_ele_sch, RB_const_ele_sch2,
            RB_const_type, RB_const_type_grund, RB_const_gt_elemente,
            RB_const_array, RB_const_array2, RB_const_array3,
            RB_const_array_liste, RB_const_array_liste2,

            RB_statement_list1, RB_statement_list, RB_statement_list2, RB_statement_list3,
            RB_statement_list4, RB_statement_list5,

            RB_zuweisung, RB_zuweisung2,
            RB_zuw_ziel_v_array_1d,
            RB_zuw_ziel_v_array_2d, RB_zuw_ziel_v_array_2d_02,

            RB_term, RB_term2, RB_term3, RB_term4, RB_term5, RB_term6,
            RB_terme1, RB_terme2,
            RB_procedure, RB_procedure2,
            RB_proc_param, RB_proc_param2, RB_proc_call, RB_proc_call_par, RB_proc_call_par2,
            RB_proc_call_par_lst, RB_proc_call_par_lst2,
            RB_proc_par_lst, RB_proc_par_lst2,
            RB_proc_VAR, RB_proc_VAR2, RB_proc_VAR3, RB_proc_VAR4,

            RB_fak_fkt, RB_fak_con_arr, RB_fak_b2w, RB_fak_low, RB_fak_high,
            RB_fak_bithigh, RB_fak_bitlow,

            RB_zuw_quelle_var_arr_1d,
            RB_zuw_quelle_var_arr_2d, RB_zuw_quelle_var_arr_2d_02,

            RB_fak_b2c, RB_fak_c2b, RB_fak_length,
            RB_fak_dwbyte, RB_fak_dwlow, RB_fak_dwhigh, RB_fak_wordtodw, RB_fak_bytetodw,

            RB_asm, RB_inc, RB_dec, RB_shl, RB_shr, RB_nop,
            RB_setbit, RB_clearbit,
            RB_function, RB_function2, RB_function3, RB_func_return,
            RB_func_call, RB_func_call_par, RB_func_call_par2,
            RB_for_to_do, RB_for_to_do2, RB_for_to_do3,
            RB_for_to_downto, RB_for_to_downto2, RB_for_to_downto3,
            RB_loop, RB_loop2,
            RB_if_then, RB_if_then2, RB_if_else, RB_if_else2,
            RB_while, RB_while2,
            RB_case, RB_case2, RB_case_liste, RB_case_liste2, RB_case_ele_typ,
            RB_case_ele_ein, RB_case_ele_ber, RB_case_ele_lis, RB_case_ele_lis2, RB_case_ele_lis3,
            RB_case_else, RB_case_else2,

            RB_isr, RB_isr2,
            RB_EnableInts, RB_DisableInts, RB_isr_liste, RB_isr_liste2,

            RB_Ende);


  RW_TR = record
    RW_SI_Unter_Typ : SI_Unter_Typ;
    RW_Call_Bez : RW_Bez;
    Ele_erf : Boolean;
  end;

  RecArray = array[1..RecArray_LEN] of RW_TR;

  RW_REC = record
    Bez : RW_Bez;
    Typ : RW_Typ;
    Rec : RecArray;
  end;


  SI_ARR_type = ARRAY [1..LEN_SI_ARR] OF SI_REC;


{$I RegelWerk.inc}




{ *** ZWISCHENCODE *** }

type

  ASM_type_Typ = (AT_kein_typ, AT_bit,
                  AT_byte_con_reg, AT_word_con_reg,
                  AT_byte_con_var, AT_word_con_var,
                  AT_boolean, AT_byte, AT_word, AT_dword,
                  AT_currency,
                  AT_char, AT_string);

  // Variablen-Type-Record
  VT_REC = record
    bezeichner : String;
    ASM_type : ASM_type_Typ;
  end;


const

   LEN_AT_CONST = 12;

  // true = $FF false = $00


{

1. biaere FKZ

  FKZ8 = +922337203685477.5807 bis
         -922337203685477.5808

  entspricht:
  Int64 - Wertebereich: (-9223372036854775808 .. 9223372036854775807)
          mit fester Kommastelle nach 4 Stellen

}

  ASM_TARR : array[1..LEN_AT_CONST] of VT_REC =
      (
  { 01 } (bezeichner : 'BYTECONREG';  ASM_type : AT_byte_con_reg),
  { 02 } (bezeichner : 'WORDCONREG';  ASM_type : AT_word_con_reg),
  { 03 } (bezeichner : 'BYTECONVAR';  ASM_type : AT_byte_con_var),
  { 04 } (bezeichner : 'WORDCONVAR';  ASM_type : AT_word_con_var),
  { 05 } (bezeichner : 'BIT';  ASM_type : AT_bit),
  { 06 } (bezeichner : 'BOOLEAN';  ASM_type : AT_boolean),
  { 07 } (bezeichner : 'BYTE';  ASM_type : AT_byte),
  { 08 } (bezeichner : 'WORD';  ASM_type : AT_word),
  { 09 } (bezeichner : 'CHAR';  ASM_type : AT_char),
  { 10 } (bezeichner : 'STRING';  ASM_type : AT_string),
  { 11 } (bezeichner : 'DWORD';  ASM_type : AT_dword),
  { 12 } (bezeichner : 'CURRENCY';  ASM_type : AT_currency)
       );

  MAX_ANZ_LOW_REG = 16;

  REG_ARRAY_LOW : array [1..MAX_ANZ_LOW_REG] of String =
      (
       { 01 } 'R0',  { 02 } 'R1',  { 03 } 'R2',  { 04 } 'R3',
       { 05 } 'R4',  { 06 } 'R5',  { 07 } 'R6',  { 08 } 'R7',
       { 09 } 'R8',  { 10 } 'R9',  { 11 } 'R10', { 12 } 'R11',
       { 13 } 'R12', { 14 } 'R13', { 15 } 'R14', { 16 } 'R15'
       );


  MAX_ANZ_HIGH_REG = 14;

  // R30 und R31 (Register Z) ist reserviert
  REG_ARRAY_HIGH : array [1..MAX_ANZ_HIGH_REG] of String =
      (
       { 01 } 'R16', { 02 } 'R17', { 03 } 'R18', { 04 } 'R19',
       { 05 } 'R20', { 06 } 'R21', { 07 } 'R22', { 08 } 'R23',
       { 09 } 'R24', { 10 } 'R25', { 11 } 'R26', { 12 } 'R27',
       { 13 } 'R28', { 14 } 'R29'
       );





type

  {
   --- Erklaerung der ANW_type Werte ---

   'X' = kein ANW_type (Vorbelegung)
   'V' = Variable (VAR)
   'Y' = Variable (ARRAY-1D)
   'Z' = Variable (ARRAY-2D)
   'C' = Konstante (CONST)
   'A' = Konstante (ARRAY)
   'B' = Bit (VAR)
   'P' = Prozedur (PROCEDURE)
   'F' = Funkion (FUNCTION)
   'L' = lokaler Uebergabeparameter innerhalb einer Prozedur oder Funktion
   'D' = lokale (dynamische) Variable (VAR) innerhalb einer Prozedur oder Funktion
   'W' = ein Zahlenwert (nicht in NAME_TAB enthalten)
   'S' = spezielle Funktion
             UnterTypen in Wert
             'W' = ByteToWord
             'L' = Low
             'H' = High
             'T' = TestBit
             'B' = ByteToChar
             'C' = CharToByte
             'S' = String-Length
             '0'..'3' = DWByte - DWord in Byte(0..3)
             'M' = DWLow  - DWord in Word - Low-Teil
             'N' = DWHigh - DWord in Word - High-Teil
             'D' = WordToDW
             'E' = ByteToDW
  }



  // statement
  statement_type =
    (STM_kein,
    { *** sonstiges *** }
    STM_stack_init, STM_Codesegment, STM_Sprungtabelle,
    STM_sprungmarke,
    // A
    STM_ADIW,
    // B
    STM_BRCC_Marke, STM_BRCS_Marke, STM_BREQ_Marke, STM_BRNE_Marke,
    STM_BRTS_Marke, STM_BRTC_Marke,
    // C
    STM_CP, STM_CPC, STM_CLI,
    // D
    STM_DEC_dword,
    STM_DEC_dword_l1, STM_DEC_dword_l2,
    // E
    STM_EIJMP_Marke, STM_EICALL, STM_ELPM,
    // I
    STM_INC_dword,
    STM_INC_dword_l1, STM_INC_dword_l2,
    STM_Interrupttabelle,
    // J
    STM_JMP_Marke,
    // M
    STM_MOV, STM_MOVW,
    // O
    STM_ORG, STM_OUT,
    // R
    STM_RET, STM_RETI,
    STM_RJMP_Marke,
    STM_ROL, STM_ROR,
    // L
    STM_LDS,
    STM_LDI, STM_LDI_word_L, STM_LDI_word_H, STM_LDI_word_3,
    STM_LSL, STM_LSR,
    // S
    STM_SBIW_1, STM_STS, STM_SEI,
    // V
    STM_VERGL_A_curr, STM_VERGL_B_curr, STM_VERGL_C_curr,



    STM_CP_byte, STM_CP_word_L, STM_CP_word_H,
    STM_CPI,  STM_LD,
    STM_PUSH, STM_POP,
    STM_PUSH_byte_A, STM_PUSH_word_AL, STM_PUSH_word_AH,
    STM_PUSH_byte_B, STM_PUSH_word_BL, STM_PUSH_word_BH,
    STM_INC_byte, STM_INC_word,
    STM_DEC_byte, STM_DEC_word,
    STM_SHL_word,
    STM_SHR_byte, STM_SHR_word,
    STM_NOP, STM_AND, STM_OR, STM_CLR, STM_SER,
    STM_LPM, STM_SBI, STM_SBR, STM_CBI, STM_CBR,
    STM_TST, STM_BST, STM_ST,
    STM_RCALL, STM_CALL,
    STM_IN_byte, STM_IN_word_L, STM_IN_word_H,


    { *** Zuweisungen Operationen *** }
    STM_zuw_op_add_byte, STM_zuw_op_add_word_L, STM_zuw_op_add_word_H,
    STM_zuw_op_sub_byte, STM_zuw_op_sub_word_L, STM_zuw_op_sub_word_H,
    STM_zuw_op_mul_byte, STM_zuw_op_mul_word, STM_zuw_op_mul_dword,
    STM_zuw_op_div_byte, STM_zuw_op_div_word, STM_zuw_op_div_dword,
    STM_zuw_op_add_curr, STM_zuw_op_sub_curr,
    STM_zuw_op_mul_curr, STM_zuw_op_div_curr,

    { *** Prozeduren *** }


   { *** ASM *** }
    STM_asm,

    { *** FOR TO DO *** }
    STM_ftd_vergl_to_byte, STM_ftd_vergl_to_word, STM_ftd_vergl_to_dword,
    STM_ftd_vergl_downto_byte, STM_ftd_vergl_downto_word, STM_ftd_vergl_downto_dword,

    { *** BOOL_CMP *** }
    STM_bo_cp,

    { *** BOOL_CMP *** }
    STM_str_const_quelle_a, STM_str_const_quelle_b,
    STM_str_var_quelle_a, STM_str_var_quelle_b, STM_str_var_quelle_char,
    STM_str_var_ziel,

    { *** ISR *** }
    STM_isr_push, STM_isr_pop,

    { *** Adresse *** }
    STM_arr_to_adress, STM_adress_to_arr


    );


  // Statement-Record
  STM_REC = record
    Statement : statement_type;
    Bemerkung, Parameter1, Parameter2 : string;
  end;



  // Name-Type-Record
  NT_REC = record
    Bezeichner, Wert : String;
    Laenge : Integer; // fuer STRING
    ASM_type, ASM_type2 : ASM_type_Typ;
    ANW_type, ANW_type2 : Char;
  end;



  // Quelle-Record
  Quelle_REC = record
    // Quelle
    Name, Wert : String;
    ANW_type, ANW_type2 : Char;
    ASM_type, ASM_Grund_type : ASM_type_Typ;
    // Operator
    Q_Operator : SI_Unter_Typ;
  end;


  // Zuweisungs-Record
  ZUW_REC = record
    // Ziel
    Ziel : String;
    Ziel_ASM_type, Ziel_ASM_Grund_type : ASM_type_Typ;
    Ziel_ANW_type, Ziel_ANW_type2 : Char;
    nur_eine_Quelle, nur_ein_Array : Boolean;
    Quelle : array [1..30] of Quelle_REC;
    QUELLcounter : Integer;
  end;




  // Zuweisungs-Liste
  ZUW_LIS_REC = record
    LISTE : array [1..10] of ZUW_REC;
    LISTcounter : Integer;
  end;





   // Procedure-Parameter-Record
  PROC_PAR_REC = record
    Parameter, Register_L, Register_H,
    Register_LH, Register_HH,
    Register_LLH, Register_LHH,
    Register_HLH, Register_HHH : string;
    ASM_type : ASM_type_Typ;
    ANW_Type : Char;
  end;


  // Procedure-Record
  PROC_REC = record
    Name : string;
    ANW_Type : Char;
    RESULT_vorhanden : Boolean;
    counter, REG_ARRAY_LOW_pointer : integer;
    PPR_ARR : array [1..Laenge_PROC_PAR_Liste] of PROC_PAR_REC;
    RESULT  : PROC_PAR_REC;
  end;


  // Procedure-Liste
  PROC_LIS_REC = record
    LISTE : array [1..LEN_PROC_LISTE] of PROC_REC;
    counter : Integer;
  end;


  // aktuelle Procedure
  akt_CALL_REC = record
    Name : string;
    PPR_ARR : array [1..Laenge_akt_CALL_Liste] of PROC_PAR_REC;
    counter, PROC_Pos : integer;
  end;


  // FOR_TO_DO-Record
  FTD_REC = record
    Ziel, Marke_Start,
    Marke_StatementList, Marke_Ende : String;
    ASM_type : ASM_type_Typ;
    Ziel_ANW_type, Vergl_ANW_type : Char;
    to_downto : Boolean;    // TO=true; DOWNTO=false;
  end;


  // FOR_TO_DO-Liste
  FTD_LIS_REC = record
    LISTE : array [1..Laenge_FTD_Liste] of FTD_REC;
    counter : Integer;
  end;



  // PROCEDURE/FUNCTION lokale Parameter
  PROC_LOK_PAR_REC = record
    Parameter,
    Register_von_L, Register_von_H,
    Register_von_LH, Register_von_HH,
    Register_von_LLH, Register_von_LHH,
    Register_von_HLH, Register_von_HHH,
    Register_nach_L, Register_nach_H,
    Register_nach_LH, Register_nach_HH,
    Register_nach_LLH, Register_nach_LHH,
    Register_nach_HLH, Register_nach_HHH,
    Register_zu_L, Register_zu_H,
    Register_zu_LH, Register_zu_HH,
    Register_zu_LLH, Register_zu_LHH,
    Register_zu_HLH, Register_zu_HHH : string;
    ASM_type : ASM_type_Typ;
    ANW_Type : Char;
    Register_kopieren : Boolean;
  end;


  // LOOP
  LOOP_REC = record
    Start_Marke : String;
  end;


  // IF_THEN
  IF_THEN_REC = record
    Start_Marke, Ende_True_Marke, Ende_Else_Marke : String;
    mit_Else : Boolean;
  end;


  // IF_THEN-Liste
  IF_THEN_LIS_REC = record
    LISTE : array [1..Lange_IF_THEN_Liste] of IF_THEN_REC;
    counter : Integer;
  end;



  // Konstanten-Array-Record
  CONST_ARRAY_REC = record
    Bezeichner : String;
    Anfang, Laenge : Word;
    ASM_type : ASM_type_Typ;
    Werte : TStringList;
  end;


  // Konstanten-Array-Liste
  CONST_ARRAY_LIS = record
    LISTE : array [1..Laenge_CONST_ARRAY_Liste] of CONST_ARRAY_REC;
    counter : Integer;
  end;



  // Konstanten-Term-Record
  CONST_TERM_REC = record
    Kennzeichner : Char;
    Wert : String;
  end;


  // Konstanten-Term-Liste
  CONST_TERM_LIS = record
    LISTE : array [1..Laenge_CONST_TERM_Liste] of CONST_TERM_REC;
    counter : Integer;
  end;




  // Variablen-Array-Record
  VAR_ARRAY_REC = record
    Bezeichner : String;
    Array_ist_1d : Boolean;
    Anfang_1d, Laenge_1d : Word;
    Anfang_2d, Laenge_2d : Word;
    ASM_type : ASM_type_Typ;
  end;


  // Variablen-Array-Liste
  VAR_ARRAY_LIS = record
    LISTE : array [1..Laenge_VAR_ARRAY_Liste] of VAR_ARRAY_REC;
    counter : Integer;
  end;


  // Bit-Record
  BIT_RECORD = record
    Bezeichner, VAR_REG : String;
    ASM_type, ASM_Grund_type : ASM_type_Typ;
    ANW_Type : Char;
    Bit_Position : Byte;
  end;


  // Bit-Liste
  BIT_LIS_REC = record
    LISTE : array [1..Laenge_Bit_Liste] of BIT_RECORD;
    counter : Integer;
  end;


  // WHILE
  WHILE_REC = record
    Vergleich_Marke, Start_Marke, Ende_Marke : String;
  end;


  // WHILE-Liste
  WHILE_LIS_REC = record
    LISTE : array [1..Laenge_While_Liste] of WHILE_REC;
    counter : Integer;
  end;



  // CASE
  CASE_REC = record
    naechstes_Element_Marke, Ende_Marke, Listen_Element_Marke,
    Wert_Original, Wert_ASM : String;
    ASM_type : ASM_type_Typ;
    erstes_Element, erstes_Listen_Element, mit_ELSE : Boolean;
  end;


  // CASE-Liste
  CASE_LIS_REC = record
    LISTE : array [1..Laenge_CASE_Liste] of CASE_REC;
    counter : Integer;
  end;



  // ISR_Zeile
  ISR_ZEILE_REC = record
    ORG_Name, ISR_Name, Bemerkung : String;
    enable : Boolean;
  end;


  // ISR_Liste
  ISR_LIS_REC = record
    LISTE : array [1..Laenge_ISR_Liste] of ISR_ZEILE_REC;
    max_anzahl, counter : Integer;
    enable : Boolean;
  end;



implementation

end.




