unit Device_Form;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  StdCtrls,

  CompilerDeklarationen, ZwischenCode;



const

  Length_ATMEGA_Array = 100;

  atmega_path = 'rp_device';
  atmega_picture_path = atmega_path + '\' + 'picture';
  atmega_pdf_path = atmega_path + '\' + 'pdf';
  atmega_fuse_path = atmega_path + '\' + 'fuse';
  atmega_array_name = 'atmega_array.txt';
  atmega_device_names = atmega_path + '\' + 'device_names.txt';

  pdf_viewer_name = 'SumatraPDF.exe';
  pdf_viewer_path = 'pdf_viewer';
  pdf_viewer_path_name = atmega_pdf_path + '\' + pdf_viewer_path + '\' + pdf_viewer_name;





type


  ATMEGA_Array_Record = record
    Bezeichner,
    RONPAS_name, AVRDUDE_name,
    INC_name, REG_name,
    Image_name, PDF_name,
    Fuses_name,
    Device_Signature : String;
    ISR_LISTE_max_Anzahl : Integer;
    ASM_Switch : ASM_Switch_Type;
  end;



  { TForm_Device }

  TForm_Device = class(TForm)
    Button_Close: TButton;
    Button_Device_PDF: TButton;
    ComboBox_ATMEGA_Typ: TComboBox;
    Image_ATMEGA: TImage;
    Label1: TLabel;
    ListBox_Meldungen: TListBox;
    procedure Button_CloseClick(Sender: TObject);
    procedure Button_Device_PDFClick(Sender: TObject);
    procedure ComboBox_ATMEGA_TypChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { private declarations }
    device_pdf_path_name : String;

    procedure ATMEGATyp_change;

  public
    { public declarations }
    ATMEGA_Array : ARRAY [0..Length_ATMEGA_Array] of ATMEGA_Array_Record;
    ATMEGA_Array_Counter : Integer;

    ATMEGA_Typ_Index : Integer;

    procedure DEFINE_NAME_IN_ATMEGA_Typ_Index(DEF_Name : String);

    procedure Read_ATMEGA_Array;

  end; { TForm_Device }

var
  Form_Device: TForm_Device;

implementation

{$R *.lfm}



uses
  Main_Unit, Modul_Compiler, Modul_Parser;


{ TForm_Prommer }


// ************************
// * INCLUDE
// ************************

{$I Device_Form_01.inc}



// ************************
// * Initialisiereung
// ************************



procedure TForm_Device.FormCreate(Sender: TObject);
var
  RP_cnt : Integer;
  RP_Line : String;
  DeviceName_list : TStringList;
begin



  device_pdf_path_name:='';


  Image_ATMEGA.Picture.Bitmap.Width:=Image_ATMEGA.Width;
  Image_ATMEGA.Picture.Bitmap.Height:=Image_ATMEGA.Height;


  // Einlesen ATmega
  Read_ATMEGA_Array;

  DeviceName_list := TStringList.Create;
  DeviceName_list.Sorted:=true;

  for RP_cnt:=0 to ATMEGA_Array_Counter-1 do begin

    RP_Line:=ATMEGA_Array[RP_cnt].Bezeichner+' ('+
             ATMEGA_Array[RP_cnt].AVRDUDE_name+')';
    ComboBox_ATMEGA_Typ.Items.Add(RP_Line);

    DeviceName_list.Add(ATMEGA_Array[RP_cnt].RONPAS_name);

  end; { for RP_cnt:=0 to Prommer_Array_Counter-1 do }


  DeviceName_list.SaveToFile(Form_RP.ronpas_path+'\'+atmega_device_names);
  DeviceName_list.Free;


  ComboBox_ATMEGA_Typ.ItemIndex := Form_RP.CB_Device_Index;

  ATMEGATyp_change;

  ATMEGA_Typ_Index:=-1;

  if FileExists(Form_RP.ronpas_path+'\'+atmega_device_names) then
    Form_RP.SynAnySyn_PAS.Objects.LoadFromFile(
                         Form_RP.ronpas_path+'\'+atmega_device_names);

end; { procedure TForm_Device.FormCreate }










// *****************************************
// *** Hilfsprogramme zur den List-Boxen ***
// *****************************************





procedure TForm_Device.ATMEGATyp_change;
var
  Image_Path_Name : String;
  Image_loaded : boolean;
begin

  Image_loaded:=false;

  if ATMEGA_Array_Counter>0 then begin

    if Length(ATMEGA_Array[ComboBox_ATMEGA_Typ.ItemIndex].Image_Name)>0 then begin

      Image_Path_Name:=Form_RP.ronpas_path+'\'+atmega_picture_path+'\'+ATMEGA_Array[ComboBox_ATMEGA_Typ.ItemIndex].Image_Name;

      if FileExists(Image_Path_Name) then begin

        Image_ATMEGA.Picture.LoadFromFile(Image_Path_Name);
        Image_loaded:=true;

      end;

    end; { if Length(Prommer_Array... }

  end;


  if Image_loaded=false then begin
    Image_ATMEGA.Picture.Bitmap.Canvas.Brush.Color:=clWhite;
    Image_ATMEGA.Picture.Bitmap.Canvas.Pen.Color:=clBlack;
    Image_ATMEGA.Picture.Bitmap.Canvas.Rectangle(0,0,Image_ATMEGA.Width,Image_ATMEGA.Height);
  end;


  device_pdf_path_name:='';
  Button_Device_PDF.Enabled:=false;
  if Length(ATMEGA_Array[ComboBox_ATMEGA_Typ.ItemIndex].PDF_name)>0 then begin

    device_pdf_path_name:=Form_RP.ronpas_path+'\'+atmega_pdf_path+'\'+
                          ATMEGA_Array[ComboBox_ATMEGA_Typ.ItemIndex].PDF_name;

    if FileExists(device_pdf_path_name) then begin
      Button_Device_PDF.Enabled:=true;
    end
    else
      device_pdf_path_name:='';

  end; { if Length(ATMEGA_Array[ComboBox_ATMEGA_Typ.ItemIndex].PDF_name)>0 then }


end; { procedure TForm_Device.ATMEGATyp_change }






procedure TForm_Device.ComboBox_ATMEGA_TypChange(Sender: TObject);
begin
  Form_RP.CB_Device_Index:=ComboBox_ATMEGA_Typ.ItemIndex;
  ATMEGATyp_change;
end; { procedure TForm_Device.ComboBox_ATMEGA_TypChange }







// ******************************************************





procedure TForm_Device.DEFINE_NAME_IN_ATMEGA_Typ_Index(DEF_Name : String);
var
  DN_Zaehler : Integer;
begin
  ATMEGA_Typ_Index:=-1;
  DEF_Name:=UpperCase(DEF_Name);
  for DN_Zaehler:=0 to ATMEGA_Array_Counter do begin
    if UpperCase(ATMEGA_Array[DN_Zaehler].RONPAS_name)=DEF_Name then
      ATMEGA_Typ_Index:=DN_Zaehler;

  end; { for TN_Zaehler:=1 to LEN_AT_CONST do }

  if ATMEGA_Typ_Index=-1 then begin
    Fehler_cl.Registrieren('Prommer/DEFINE_NAME_IN_uC','der DEFINE-Name des Microcontrollers ('+
                           DEF_Name+') wird nicht unterstuetzt',
                           Compiler_cl.SI_ARR[Parser_cl.SI_ARR_PAR_zaehler].datei,
                           Compiler_cl.SI_ARR[Parser_cl.SI_ARR_PAR_zaehler].zeile,
                           Compiler_cl.SI_ARR[Parser_cl.SI_ARR_PAR_zaehler].spalte);

  end { else ATMEGA_Typ_Index=-1 then }
  else begin

    ComboBox_ATMEGA_Typ.ItemIndex:=ATMEGA_Typ_Index;

  end; { if ATMEGA_Typ_Index=-1 then }

end; { procedure TForm_Prommer.DEFINE_NAME_IN_uC }







// ****************************************************
// *** PDF-anzeigen
// ****************************************************


procedure TForm_Device.Button_Device_PDFClick(Sender: TObject);
var
  pdf_program_path_name : String;
begin

  if Length(device_pdf_path_name)>0 then begin

    pdf_program_path_name:=Form_RP.ronpas_path+'\'+pdf_viewer_path_name;

    SysUtils.ExecuteProcess( AnsiString(pdf_program_path_name),
                             device_pdf_path_name, []);

  end; { if Length(device_pdf_path_name)>0 then }

end; { procedure TForm_Device.Button_Device_PDFClick }







// *** Ende ***

procedure TForm_Device.Button_CloseClick(Sender: TObject);
begin
  Close;
end; { procedure TForm_Device.Button_CloseClick }






procedure TForm_Device.FormDestroy(Sender: TObject);
begin
  Close;
end; { procedure TForm_Device.FormDestroy }





end.

