unit Modul_Assembler;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Windows;


const

  assembler_name = 'avrasm2.exe';
  assembler_path = 'rp_assembler';
  assembler_bat_name = 'assembling.bat';
  assembler_include_path = assembler_path + '\' + 'include';

  register_path = 'rp_register';

  Buffer_Size = 1000000;


  BUF_SIZE = 2048; // Buffer size for reading the output in chunks



type

  Assembler_class = class(TObject)
  public
    constructor Create;
    procedure Start_Assembler;
    destructor Destroy; override;
  private
    procedure RunCaptured(var Ausgabe: TStringList;
                          const _dirName, _exeName: string);
    procedure Run_BATCH_Datei(const dirName, batName: string);
  end;

var
  Assembler_cl : Assembler_class;

implementation


uses
  Main_Unit;




constructor Assembler_class.Create;
begin
  inherited Create;

end;  { constructor Assembler_class.Create }








procedure Assembler_class.RunCaptured(var Ausgabe: TStringList;
                                      const _dirName, _exeName: string);
const
  max_read=3000;

var
  StartupInfo: TStartupInfo;
  ProcessInfo: TProcessInformation;
  Buffer: array[0..10000] of char;
  bRead: DWord;
  hRead, hWrite: THandle;
  saAttr: TSECURITYATTRIBUTES;
  Output: TMemoryStream;
  exitcode: cardinal;
  AExitCode: boolean;
  Prozess_laeuft : Boolean;
  WarteZeit : Integer;


begin
  AExitCode:=False;

  saAttr.nLength := sizeof(TSECURITYATTRIBUTES);
  saAttr.bInheritHandle := true;
  saAttr.lpSecurityDescriptor := nil;
  if not CreatePipe(hRead, hWrite, @saAttr, 0) then
  begin
    Ausgabe.Add('Fehler: Die Pipe kann nicht erzeugt werden!');
    Exit;
  end;
  try
    FillChar(StartupInfo, Sizeof(StartupInfo), #0);
    StartupInfo.cb := Sizeof(StartupInfo);
    StartupInfo.dwFlags := STARTF_USESHOWWINDOW or STARTF_USESTDHANDLES;
    StartupInfo.wShowWindow := SW_HIDE and SW_SHOWMINNOACTIVE;
    { Handle mit Childhandle Assoziieren }
    StartupInfo.hStdInput := GetStdHandle(STD_INPUT_HANDLE);
    { Ausgaben abfangen }
    StartupInfo.hStdOutput := hWrite;
    { Fehler abfangen }
    StartupInfo.hStdError := hWrite;

    if not CreateProcess(nil, PChar(_exeName), nil, nil, True,
      0, nil, PChar(_dirName), StartupInfo, ProcessInfo) then
    begin
      Ausgabe.Add('Fehler: Der Prozess kann nicht erzeugt werden!');
      Ausgabe.Add('ExeName='+_exeName);
      Ausgabe.Add('DirName='+_dirName);
    end
    else
    begin
      { loop bis zum Abbruch }

      Prozess_laeuft:=true;
      WarteZeit:=0;
      while Prozess_laeuft do begin

        if WaitforSingleObject(ProcessInfo.hProcess, 0) = WAIT_OBJECT_0 then
          Prozess_laeuft:=false;


      end; { while Prozess_laeuft do }

      GetExitCodeProcess(ProcessInfo.hProcess, exitcode);

      if AExitCode = False then begin
        { Den gesamten Output lesen und in eine StringList kopieren }
        Output := TMemoryStream.Create;
        Output.Clear;
        repeat
          Buffer := #0;

          if ReadFile(hRead, Buffer, max_read, bRead, nil) then begin
            Output.WriteBuffer(Buffer, bRead);
            Output.Position := bRead;
          end
          else
            break;

        until bRead <> max_read;

        Output.Position := 0;
        Buffer := #0;
        { Die Ausgaben hinzufuegen }

        if output.Size<Buffer_Size then
          output.Read(Buffer, output.Size)
        else
          output.Read(Buffer, Buffer_Size-10);

        Ausgabe.Text:=Buffer;
        FreeAndNil(Output);

      end
      else
        Ausgabe.Add('FEHLER: mit Fehlercode='+IntToStr(exitcode));

    end;
  finally
    { Lese- und Schreibhandles der Pipe schliessen }
    CloseHandle(hRead);
    CloseHandle(hWrite);
  end;
end; { procedure Assembler_class.RunCaptured }









procedure Assembler_class.Run_BATCH_Datei(const dirName, batName: string);
var
  BATCH_String : String;
begin
  BATCH_String:=dirName+batName;

  SysUtils.ExecuteProcess(PChar(BATCH_String), '', []);

end; { procedure Assembler_class.Run_BATCH_Datei }








procedure Assembler_class.Start_Assembler;
var
  bat_pfad, asm_temp_path,
  bsp_pfad, bsp_name, zeile_list,
  list_file  : String;
  Ausgabe2 : TStringList;
  zaehler : Integer;



  procedure Run_Assembler(asm_error : Boolean);
  var
    zeile_command, zeile_asm,
    a1, a2, a3, a4, a5, a6 : String;
    bat_datei : TStringList;
  begin


    a1 := assembler_name;
    a2 := ' -fI -W+ie -C V2E -o "' + bsp_pfad + bsp_name + '.hex"';
    a3 := ' -e "' + bsp_pfad + bsp_name + '.eep"';
    a4 := ' -I "' + asm_temp_path+assembler_include_path+'"';
    a5 := ' -l "'+ list_file +'"';
    a6 := ' "' + bsp_pfad + bsp_name + '.asm"';
    zeile_asm:= a1 + a2 + a3 + a4 + a5 + a6;


    bat_datei := TStringList.Create;


    if asm_error then begin

      zeile_command:='echo on';
      bat_datei.Add(zeile_command);

      zeile_command:='color f0';
      bat_datei.Add(zeile_command);

      zeile_command:='mode con: cols=120 lines=400';
      bat_datei.Add(zeile_command);

    end;



    zeile_command:='cd '+assembler_path;
    bat_datei.Add(zeile_command);


    zeile_command:='Set path='+
                   asm_temp_path+assembler_include_path+';'+
                   asm_temp_path+register_path+';'+
                   '%path%';
    bat_datei.Add(zeile_command);


    bat_datei.Add(zeile_asm);


    if asm_error then begin

      zeile_command:='pause';
      bat_datei.Add(zeile_command);

    end;


    bat_datei.SaveToFile(Form_RP.ronpas_path+'\'+assembler_bat_name);

    bat_datei.Free;

    Run_BATCH_Datei(bat_pfad,assembler_bat_name);


  end; { procedure Run_Assembler }



begin


  Form_RP.ListBox_Meldungen.Items.Add('Assembler wurde gestartet ...');

  Form_RP.Image_COMPILE(ic_gelb);

  bsp_pfad := Form_RP.source_path+'\';
  bsp_name := Form_RP.source_name;

  asm_temp_path:=Form_RP.ronpas_path+'\';
  bat_pfad:=asm_temp_path;

  list_file := bsp_pfad + bsp_name + '.lst';



  if FileExists(list_file) then
    DeleteFile(PChar(list_file));


  Run_Assembler(false);


  if FileExists(list_file) then begin

    Ausgabe2:=TStringList.Create;

    RunCaptured(Ausgabe2,bat_pfad,assembler_bat_name);

    for zaehler:=0 to Ausgabe2.Count-1 do begin
      zeile_list:= Ausgabe2[zaehler];
      Form_RP.ListBox_Meldungen.Items.Add(zeile_list);
    end;

    Ausgabe2.Free;

    if not Form_RP.CheckBox_LOG_Dateien then
      DeleteFile(PChar(list_file));


    Form_RP.Image_COMPILE(ic_gruen);


  end
  else begin

    Run_Assembler(true);
    Form_RP.ListBox_Meldungen.Items.Add('*** Fehler beim assemblieren ***');

    Form_RP.Image_COMPILE(ic_rot);

  end;

  DeleteFile(PChar(Form_RP.ronpas_path+'\'+assembler_bat_name));

end; { procedure Assembler_class.Start_Assembler }

















destructor Assembler_class.Destroy;
begin

  inherited Destroy;
end;











end.

